@@ -3,25 +3,25 @@
 # -*- coding: utf-8 -*-
 # create-upstream-tarballs - Utility to create the required source tarballs
 #                            to package Thunderbird within Debian
-# Copyright (c) 2022-2023 Carsten Schoenert <c.schoenert@t-online.de>
+# Copyright (c) 2022-2024 Carsten Schoenert <c.schoenert@t-online.de>
 #
 # SPDX-License-Identifier: GPL-2.0-or-later
 
 import argparse
 import logging
 import os
-import requests
 import shutil
 import sys
 import tarfile
-
-from lxml import html
 from zipfile import ZipFile
 
+import requests
+from lxml import html
+
 try:
-    from pkg_resources import parse_version
+    from packaging.version import Version
 except ImportError:
-    print("pkg_resources library missing, please install python3-pkg-resources!")
+    print("pkg_resources library missing, please install python3-packaging!")
     sys.exit(1)
 
 try:
@@ -33,9 +33,26 @@ except ImportError:
 # local stuff
 import repack
 
+TB_BASE_URL_RELEASES = "https://ftp.mozilla.org/pub/thunderbird/releases/"
+TB_BASE_URL_CANDIDATES = "https://ftp.mozilla.org/pub/thunderbird/candidates/"
 
-tb_base_url_releases = "https://ftp.mozilla.org/pub/thunderbird/releases/"
-tb_base_url_candidates = "https://ftp.mozilla.org/pub/thunderbird/candidates/"
+# Some strings we need to exclude within the detection of the current
+# release version.
+RELEASE_STRING_EXCLUSION_LIST = (
+ "-real",
+ "125.0",
+ "127.0",
+ "129.0",
+)
+
+# Also we need to exclude some strings in the current planned version
+# detection.
+PLANNED_STRING_EXCLUSION_LIST = (
+ "124.0",
+ "125.0",
+ "127.0",
+ "129.0",
+)
 
 try:
     import colorlog
@@ -107,7 +124,7 @@ def option_parser():
         "--repack",
         action="store_true",
         help="Strip out the unwanted and non needed files and create "
-        "the thunderbird*.orig.tar.xz tarball, requires optional paramater -v.",
+        "the thunderbird*.orig.tar.xz tarball, requires optional parameter -v.",
     )
 
     parser.add_argument(
@@ -126,10 +143,10 @@ def get_website_data(site):
     Get the raw HTML data from 'site' and returns the HTML dom data.
     """
     try:
-        page_data = requests.get(site)
+        page_data = requests.get(site, timeout=10)
     except requests.exceptions.ConnectionError as error:
         logging.error("Connection to Mozilla CDN not possible!")
-        logging.debug(f"The following error did happen:\n{error}")
+        logging.debug("The following error did happen:\n%s", error)
         sys.exit(1)
 
     if page_data.status_code == requests.codes.ok:
@@ -145,7 +162,7 @@ def get_latest_candidates_build(url):
     <a href=...> elements and split of the path from href.
     Finally count all found entries so we know the highest available build version.
     """
-    logging.debug(f"Got URL {url}")
+    logging.debug("Got URL %s", url)
     candidates_html = get_website_data(url)
     candidates_href_entries = candidates_html.xpath("//a /@href")
     latest_build = 0
@@ -156,7 +173,7 @@ def get_latest_candidates_build(url):
                 build_count = int(line[5].strip("build"))
             if build_count > latest_build:
                 latest_build = build_count
-    logging.debug(f"Found folder 'build{latest_build}/' as most recent build.")
+    logging.debug("Found folder 'build%s/' as most recent build.", latest_build)
     return latest_build
 
 
@@ -181,7 +198,7 @@ def get_versions():
     #
     # Crawling data containing the final candidates (ESR) versions. New
     # planned ESR versions will be first published and are visible here!
-    tb_release_candidates_html = get_website_data(tb_base_url_candidates)
+    tb_release_candidates_html = get_website_data(TB_BASE_URL_CANDIDATES)
 
     if tb_release_candidates_html != "None":
         tb_href_entries = tb_release_candidates_html.xpath("//a /@href")
@@ -194,6 +211,8 @@ def get_versions():
                 len(line) == 6
                 and line[3] == "candidates"
                 and line[4].endswith("candidates")
+                # Exclude all "wrong" versions which never will become a ESR version.
+                and not line[4].startswith(PLANNED_STRING_EXCLUSION_LIST)
                 and "b" not in line[4]
             ):
                 # We have now all versions with '-candidates' as suffix which
@@ -201,20 +220,20 @@ def get_versions():
                 tb_upstream_candidates_versions.append(line[4].split("-")[0])
 
     else:
-        logging.error(f"Could not load versions from '{tb_base_url_candidates}'!")
+        logging.error("Could not load versions from '%s'!", TB_BASE_URL_CANDIDATES)
         sys.exit(1)
 
     release_planned_version = tb_upstream_candidates_versions[1]
     for pos in range(2, len(tb_upstream_candidates_versions)):
-        check = parse_version(release_planned_version) < parse_version(
-            tb_upstream_candidates_versions[pos]
+        check = Version(release_planned_version.replace('esr', '')) < Version(
+            tb_upstream_candidates_versions[pos].replace('esr', '')
         )
         if check:
             release_planned_version = tb_upstream_candidates_versions[pos]
 
     # Now searching the newest build for the found candidates version.
     release_candidate_build = get_latest_candidates_build(
-        f"{tb_base_url_candidates}{release_planned_version}-candidates/"
+        f"{TB_BASE_URL_CANDIDATES}{release_planned_version}-candidates/"
     )
     release_candidate_version = release_planned_version
 
@@ -222,7 +241,7 @@ def get_versions():
     #
     # Working on data containing the final released (ESR) versions. Found
     # versions here are really released as officially.
-    tb_releases_html = get_website_data(tb_base_url_releases)
+    tb_releases_html = get_website_data(TB_BASE_URL_RELEASES)
 
     if tb_releases_html != "None":
         tb_href_entries = tb_releases_html.xpath("//a /@href")
@@ -232,16 +251,17 @@ def get_versions():
             # Processing all href elements and collect all of them with a numeric version.
             line = elem.split("/")
             if len(line) == 6:
-                # Exclude all non numeric elements.
+                # Exclude all non numeric elements. But also all versions
+                # which never will become a ESR version.
                 if line[4] not in [
                     "custom-updates",
                     "latest",
                     "latest-beta",
-                ] and not line[4].endswith(("esr", "-real")):
+                ] and not line[4].endswith(RELEASE_STRING_EXCLUSION_LIST):
                     tb_upstream_versions.append(line[4])
 
     else:
-        logging.error(f"Could not load versions from '{tb_base_url_releases}'!")
+        logging.error("Could not load versions from '%s'!", TB_BASE_URL_RELEASES)
         sys.exit(1)
 
     # Step 3 - Find most recent Release and Beta versions.
@@ -250,22 +270,22 @@ def get_versions():
     beta_version = release_version
     for pos in range(2, len(tb_upstream_versions)):
         if "b" not in tb_upstream_versions[pos]:
-            check = parse_version(release_version) < parse_version(
-                tb_upstream_versions[pos]
+            check = Version(release_version.replace('esr', '')) < Version(
+                tb_upstream_versions[pos].replace('esr', '')
             )
             if check:
                 release_version = tb_upstream_versions[pos]
 
         if "b" in tb_upstream_versions[pos]:
-            check = parse_version(beta_version) < parse_version(
+            check = Version(beta_version) < Version(
                 tb_upstream_versions[pos]
             )
             if check:
                 beta_version = tb_upstream_versions[pos]
 
-    logging.debug(f"Current Release version: {release_version}")
-    logging.debug(f"Current (planned) Release version: {release_candidate_version} (build{release_candidate_build})")
-    logging.debug(f"Current Beta version: {beta_version}")
+    logging.debug("Current Release version: %s", release_version)
+    logging.debug("Current (planned) Release version: %s (build%s)", release_candidate_version, release_candidate_build)
+    logging.debug("Current Beta version: %s", beta_version)
     return [
         release_version,
         release_candidate_version,
@@ -278,7 +298,7 @@ def get_xpi_languages(base_url):
     """
     Picks up the available XPI languages from a given URL.
     """
-    logging.debug(f"Using {base_url} to search for xpi files.")
+    logging.debug("Using %s to search for xpi files.", base_url)
     xpi_html = get_website_data(base_url)
     # Try to get xpath data elements, will be successful if the URL is a
     # valid and usable URL for the requested version! Version to download
@@ -286,7 +306,7 @@ def get_xpi_languages(base_url):
     try:
         xpi_href_entries = xpi_html.xpath("//a /@href")
     except AttributeError:
-        logging.error(f"Requested version not found on '{base_url}'!")
+        logging.error("Requested version not found on '%s'!", base_url)
         logging.error("Is the version really available on the release channel?")
         sys.exit(1)
     xpi_list = []
@@ -307,7 +327,7 @@ def get_xpi_languages(base_url):
                 xpi_list.append(line[8])
 
     if len(xpi_list) > 0:
-        logging.info(f"Found {len(xpi_list)} xpi files on remote side.")
+        logging.info("Found %d xpi files on remote side.", len(xpi_list))
         return xpi_list
     else:
         logging.error("Something went wrong while collecting XPI languages!")
@@ -350,23 +370,23 @@ def download_file(url, file, target_folder):
     Download a given file.
     If file is already locally available check if a new download is required.
     """
-    with requests.get(url, stream=True) as r:
+    with requests.get(url, stream=True, timeout=10) as r:
         # Check header to get content length in bytes.
         total_length = int(r.headers.get("Content-Length"))
         if os.path.isfile(f"{target_folder}/{file}"):
             file_size = os.path.getsize(f"{target_folder}/{file}")
             if total_length != file_size:
                 logging.warning(
-                    f"Found file '{file}' locally, but existing file size differs! Removing!"
+                    "Found file '%s' locally, but existing file size differs! Removing!", file
                 )
                 os.unlink(f"{target_folder}/{file}")
             else:
                 logging.debug(
-                    f"Found file '{file}' locally and file size matches upstream size, skipping download."
+                    "Found file '%s' locally and file size matches upstream size, skipping download.", file
                 )
                 return
 
-        logging.info(f"Download {url}")
+        logging.info("Download %s", url)
         with tqdm.wrapattr(
             r.raw, "read", total=total_length, ncols=100, desc="{:9s}".format(file)
         ) as raw:
@@ -384,24 +404,24 @@ def collect_tb_upstream_data(tb_version):
         file = f"thunderbird-{tb_version}.source.tar.xz"
         # Concatenate URLs for ESR or Beta version.
         if "b" not in tb_version:
-            tb_url = f"{tb_base_url_releases}{tb_version}/source/{file}"
-            xpi_url = f"{tb_base_url_releases}{tb_version}/linux-x86_64/xpi"
+            tb_url = f"{TB_BASE_URL_RELEASES}{tb_version}/source/{file}"
+            xpi_url = f"{TB_BASE_URL_RELEASES}{tb_version}/linux-x86_64/xpi"
         else:
             build_nb = get_latest_candidates_build(
-                f"{tb_base_url_candidates}{tb_version}-candidates/"
+                f"{TB_BASE_URL_CANDIDATES}{tb_version}-candidates/"
             )
-            tb_url = f"{tb_base_url_candidates}{tb_version}-candidates/build{build_nb}/source/{file}"
-            xpi_url = f"{tb_base_url_candidates}{tb_version}-candidates/build{build_nb}/linux-x86_64/xpi"
+            tb_url = f"{TB_BASE_URL_CANDIDATES}{tb_version}-candidates/build{build_nb}/source/{file}"
+            xpi_url = f"{TB_BASE_URL_CANDIDATES}{tb_version}-candidates/build{build_nb}/linux-x86_64/xpi"
     else:
         file = f"thunderbird-{tb_version[0]}.source.tar.xz"
         # Concatenate URLs for planned ESR version.
-        tb_url = f"{tb_base_url_candidates}{tb_version[0]}-candidates/build{tb_version[1]}/source/{file}"
-        xpi_url = f"{tb_base_url_candidates}{tb_version[0]}-candidates/build{tb_version[1]}/linux-x86_64/xpi"
+        tb_url = f"{TB_BASE_URL_CANDIDATES}{tb_version[0]}-candidates/build{tb_version[1]}/source/{file}"
+        xpi_url = f"{TB_BASE_URL_CANDIDATES}{tb_version[0]}-candidates/build{tb_version[1]}/linux-x86_64/xpi"
 
     xpi_languages = get_xpi_languages(f"{xpi_url}/")
 
     if not os.path.isdir(download_folder):
-        logging.info(f"Create folder {download_folder}")
+        logging.info("Create folder %s", download_folder)
         os.mkdir(download_folder)
 
     # Getting Thunderbird sources.
@@ -426,20 +446,20 @@ def create_tb_l10n_tarball(xpi_languages, base_folder, upstream_version):
                 if entry.is_dir(follow_symlinks=False) or entry.is_file():
                     if entry.is_dir():
                         logging.debug(
-                            f"Removing folder {l10n_upstream_folder}/{entry.name}"
+                            "Removing folder %s/%s", l10n_upstream_folder, entry.name
                         )
                         shutil.rmtree(f"{l10n_upstream_folder}/{entry.name}")
                     if entry.is_file():
                         logging.debug(
-                            f"Removing file {l10n_upstream_folder}/{entry.name}"
+                            "Removing file %s/%s", l10n_upstream_folder, entry.name
                         )
                         os.unlink(f"{l10n_upstream_folder}/{entry.name}")
         else:
-            logging.debug(f"'{l10n_upstream_folder}' exists, but nothing to clean up.")
+            logging.debug("'%s' exists, but nothing to clean up.", l10n_upstream_folder)
 
     else:
         logging.debug(
-            f"Create folder '{l10n_upstream_folder}' for xpi l10n upstream data."
+            "Create folder '%s' for xpi l10n upstream data.", l10n_upstream_folder
         )
         os.mkdir(l10n_upstream_folder)
 
@@ -447,9 +467,9 @@ def create_tb_l10n_tarball(xpi_languages, base_folder, upstream_version):
     for xpi in xpi_languages:
         l10n_folder = f"{l10n_upstream_folder}/" + xpi.split(".")[0]
         l10n_xpi_file = f"{base_folder}/{xpi}"
-        logging.debug(f"Create folder {l10n_folder}")
+        logging.debug("Create folder %s", l10n_folder)
         os.mkdir(l10n_folder)
-        logging.debug(f"Extract data from {l10n_xpi_file}")
+        logging.debug("Extract data from %s", l10n_xpi_file)
         with ZipFile(l10n_xpi_file, "r") as zipfile:
             zipfile.extractall(l10n_folder)
 
@@ -461,7 +481,7 @@ def create_tb_l10n_tarball(xpi_languages, base_folder, upstream_version):
         version = f"{upstream_version[0]}"
 
     l10n_component_name = f"thunderbird_{version}.orig-thunderbird-l10n.tar.xz"
-    logging.info(f"Build l10n component tarball {l10n_component_name}")
+    logging.info("Build l10n component tarball %s", l10n_component_name)
 
     # Build the component tarball with the l10n data.
     with tarfile.open(f"{download_folder}/../{l10n_component_name}", "w:xz") as tar:
@@ -481,7 +501,7 @@ def compare_xpi_languages(l10n_languages_remote):
     l10n_languages_local_control = []
 
     # Get a list l10n packages from debian/control.
-    with open(os.path.join(os.path.dirname(__file__), "control")) as control:
+    with open(os.path.join(os.path.dirname(__file__), "control"), encoding="utf-8") as control:
         lines = control.readlines()
         for line in lines:
             if line.startswith("Package: thunderbird-l10n-") and "-all" not in line:
@@ -494,12 +514,12 @@ def compare_xpi_languages(l10n_languages_remote):
 
         if l10n_language_remote not in l10n_languages_local:
             logging.warning(
-                f"Found language '{l10n_language_remote}' within upstream data, but not in folder thunderbird-l10n/ !"
+                "Found language '%s' within upstream data, but not in folder thunderbird-l10n/ !", l10n_language_remote
             )
 
         if l10n_language_remote.lower() not in l10n_languages_local_control:
             logging.warning(
-                f"Found language '{l10n_language_remote}' within upstream data, but not in debian/control !"
+                "Found language '%s' within upstream data, but not in debian/control !", l10n_language_remote
             )
 
 
@@ -508,13 +528,13 @@ def do_repack(source, target, filter_elements):
     Takes the source tarball and filter out all the given elements into the
     target tarball.
     """
-    logging.debug(f"Create {target} from {source} using {filter_elements}.")
+    logging.debug("Create %s from %s using %s.", target, source, filter_elements)
     repack.filter_tar(source, target, filter_elements)
 
 
 if __name__ == "__main__":
     create_logger()
-    logging.info(f"{os.path.basename(sys.argv[0])} started")
+    logging.info("%s started", os.path.basename(sys.argv[0]))
 
     # The base path the tarballs will get placed finally.
     tarball_path = os.path.dirname(os.path.abspath("./"))
@@ -523,7 +543,7 @@ if __name__ == "__main__":
 
     if args.debug:
         logging.getLogger().setLevel(logging.DEBUG)
-    logging.debug(f"Using args: {args}")
+    logging.debug("Using args: %s", args)
 
     if not args.filter:
         args.filter = os.path.join(os.path.dirname(__file__), "source.filter")
@@ -545,13 +565,13 @@ if __name__ == "__main__":
     if args.get_versions:
         upstream_versions = get_versions()
         logging.info(
-            f"Current most recent TB Release ESR version:       {upstream_versions[0]}"
+            "Current most recent TB Release ESR version:       %s", upstream_versions[0]
         )
         logging.info(
-            f"Current most recent TB Release candidate version: {upstream_versions[1]} build{upstream_versions[2]}"
+            "Current most recent TB Release candidate version: %s build%s", upstream_versions[1], upstream_versions[2]
         )
         logging.info(
-            f"Current most recent TB Beta version:              {upstream_versions[3]}"
+            "Current most recent TB Beta version:              %s", upstream_versions[3]
         )
         sys.exit(0)
 
@@ -566,7 +586,7 @@ if __name__ == "__main__":
             version = target_version[0]
 
     download_folder = f"../tb-preparation-{version}"
-    logging.debug(f"Upstream version to get: {version}")
+    logging.debug("Upstream version to get: %s", version)
 
     # Download Thunderbird source tarball and also the upstream data of l10n
     # languages Add-ons.
@@ -587,14 +607,14 @@ if __name__ == "__main__":
         source = f"{download_folder}/thunderbird-{version}.source.tar.xz"
 
     target = f"{tarball_path}/thunderbird_{version}.orig.tar.xz"
-    logging.info(f"Build source tarball thunderbird_{version}.orig.tar.xz")
+    logging.info("Build source tarball thunderbird_%s.orig.tar.xz", version)
     do_repack(source, target, args.filter)
 
     logging.info(
-        f"Thunderbird source tarball prepared as:         {tarball_path}/thunderbird_{version}.orig.tar.xz"
+        "Thunderbird source tarball prepared as:         %s/thunderbird_%s.orig.tar.xz", tarball_path, version
     )
     logging.info(
-        f"Thunderbird l10n component tarball prepared as: {tarball_path}/{tb_l10n_component_name}"
+        "Thunderbird l10n component tarball prepared as: %s/%s", tarball_path, tb_l10n_component_name
     )
 
     # Some checking if further adjustments are required to have packaging in
